/* SyncDateToName - Adjust file timestamp to match date embedded in filename

   Copyright (c) 2015-2022 Steven Levine and Associates, Inc.
   All rights reserved.

   2015-02-13 SHL Baseline - clone from elsewhere
   2016-02-07 SHL Scan smarter
   2016-02-08 SHL Handle quotes better
   2020-01-15 SHL Convert to globals
   2020-01-15 SHL Sync with templates
   2020-12-19 SHL Sync with templates
   2020-12-19 SHL Support future dates better
   2021-09-10 SHL Sync with templates - AskYNQ
   2022-04-19 SHL Sync with templates
   2022-04-19 SHL Support file names starting with file timestamp
   2022-04-19 SHL Support file names with whitespace following timestamp
*/

signal on Error
signal on Failure name Error
signal on Halt
signal on NotReady name Error
signal on NoValue name Error
signal on Syntax name Error

gVersion = '0.1 2022-04-19'

Globals = 'gArgList. gCmdName gCurYear gDbgLvl gEnv gVerbose gVersion'

call Initialize

Main:
  parse arg cmdLine
  call ScanArgs cmdLine
  drop cmdLine

  numeric digits 14

  gCurYear = left(date('S'), 4)

  call DoWildArgs

  exit

/* end main */

/*=== DoWildArgs() Process wildcarded argument list ===*/

DoWildArgs: procedure expose (Globals)

  do argNum = 1 to gArgList.0
    wildCard = gArgList.argNum
    wildCard = strip(wildCard,, '"')
    call DbgMsg 'Searching for' wildCard

    /* Format options F: files D: directories S: recurse */
    /* TL: YYYY-MM-DD HH:MM:SS Size ADHRS Name */
    call SysFileTree wildCard, 'fileList', 'FTL'
    if RESULT \= 0 then
      call Die 'SysFileTree' wildCard 'failed'
    do fileNum = 1 to FileList.0
      call DoOneFile FileList.fileNum
    end /* fileNum */
  end /* argNum */

  return

/* end DoWildArgs */

/*=== DoOneFile(fileinfo) Process one file ===*/

DoOneFile: procedure expose (Globals)

  parse arg fileInfo
  parse var fileinfo fileDate fileTime fileBytes fileAttrib pathName
  pathName = strip(pathName)

  call DbgMsg 'Checking' pathName

  i = lastpos('\', pathName)

  if i = 0 then
    call Die 'Can not find \ in' pathname

  filename = substr(pathname, i + 1)

  i = lastpos('.', filename)
  if i = 0 then
    call Die 'Can not find . in' filename

  ext = substr(filename, i)

  basename = left(filename, i - 1);

  call DbgMsg 'basename' basename

  shortname = basename || ext

  /* Find date in basename
     Support mm-dd-yy, mm-dd-yyyy, yyyy-mm-dd
     Allow prefix- and -suffix
   */

  /* Find first digit in basename */
  /* If part of document name, assume will not look like timestamp unless it is timestamp */
  offset = verify(basename, '0123456789', 'M')
  skip = 0

  s = translate(basename, '-', '_')	/* Normalize separators */

  do while offset > 0

    call DbgMsg 'offset' offset 'skip' skip

    if skip then do
      skip = 0
      /* Find 1st non-digit */
      offset = verify(basename, '0123456789', 'N', offset)
    end

    if offset = 0 then
      leave				/* Can't have date */

    /* Find next digit */
    offset = verify(basename, '0123456789', 'M', offset)
    if offset = 0 then
      leave				/* Can't have date */

    /* Try for mm-dd-yy or yyyy-mm-dd-foo */
    parse var s prefix =(offset) mm'-'dd'-'yy'-'suffix
    ymd = ComposeDate(mm, dd, yy)
    if ymd \== '' then leave

    /* Try for mm-dd-yy or yyyy-mm-dd foo */
    parse var s prefix =(offset) mm'-'dd'-'yy suffix
    ymd = ComposeDate(mm, dd, yy)
    if ymd \== '' then leave

    /* Try for mm-dd-yy or yyyy-mm-dd */
    parse var s prefix =(offset) mm'-'dd'-'yy
    ymd = ComposeDate(mm, dd, yy)
    if ymd \== '' then leave

    /* Try for yyyymmdd */
    parse var s prefix =(offset) yy +4 mm +2 dd +2
    ymd = ComposeDate(mm, dd, yy)
    if ymd \== '' then leave

    skip = 1				/* Find next possibl*/
  end /* do */

  if offset = 0 then do
    say 'Can not parse date from' basename '- skipping'
    return
  end

  call DbgMsg 'ymd' ymd, 'fileDate' fileDate

  /* If date adjust needed, prompt to change */

  if ymd == fileDate then
    say 'File timestamp matches pdf timestamp' ymd
  else do
    say
    say shortname 'date is' fileDate
    call AskYNQ 'Change date to' ymd
    if RESULT >= 2 then
      exit
    if RESULT = 0 then do
       err = SysSetFileDateTime(pathName, ymd, '00:00:00')
       if err \= 0 then
	 call Die 'Can not set date'
    end
  end

  return

/* end DoOneFile */

/*=== GetStatementName(basename) Process argument ===*/

GetStatementName: procedure expose (Globals)

  parse arg basename

  /* Rename from Statement0930047783
     to	Statement-09-30-04-7783
  */

  /* FIXME to check already renamed */
  newbase = basename

  do 1
    c = length(basename)

    if c \= 19 then
      leave

    if substr(basename, 1, 9) \== 'Statement' then
      leave
    s = substr(basename, 10)
    if \ datatype(s, 'W') then
      leave

    newbase = translate('ABCDEFGHI-JK-LM-NO-PQRS',basename,'ABCDEFGHIJKLMNOPQRS')
  end

  return newbase

/* end GetStatementName */

/*=== GetTaxFormName(basename) Process argument ===*/

GetTaxFormName: procedure expose (Globals)

  parse arg basename
  /* Rename from AccountSummaries20070222087783
     to	AccountSummaries-2007-02-22-08-7783
  */

  /* FIXME to check already renamed */
  newbase = basename

  if 0 then trace '?A'

  do 1
    if left(basename, 4) == '1099' then
      i = verify(basename, '0123456789', 'M', 5)
    else if left(basename, 4) == '5498' then
      i = 5
    else do
      i = verify(basename, '0123456789', 'M')
    end

    if i = 0 then
      leave

    s = substr(basename, i)

    if \ datatype(s, 'W') then
      leave

    newtail = translate('ABCD-EF-GH-IJ-KLMN',s,'ABCDEFGHIJKLMN')

    newbase = left(basename, i - 1) || '-' || newtail
  end

  call DbgMsg 'newbase' newbase

  return newbase

/* end GetTaxFormName */

/*=== ComposeDate(mm, dd, yy) Return yyyy-mm-dd or empty string ===*/

ComposeDate: procedure expose (Globals)

 /* Input is mm-dd-yy or yyyy-mm-dd-foo */

  parse arg mm, dd, yy

  call DbgMsg 'Checking mm' mm 'dd' dd 'yy' yy

  ymd = ''
  if datatype(mm, 'W') & datatype(dd, 'W') & datatype(yy, 'W') then do
     /* Check if really yyyy-mm-dd */
     if mm > 1900 then do
       y1 = mm
       m1 = dd
       dd = yy
       yy = y1
       mm = m1
     end
    /* Extend year to 4 digits */
    if yy >= 80 & yy <= 100 then
      yy = yy + 1900
    else if yy < 80 then
      yy = yy + 2000
    /* Allow current year plus 1 in case working in December */
    if yy >= 1900 & yy <= gCurYear + 1 & mm >= 1 & mm <= 12 & dd >= 1 & dd <= 31 then
      ymd = yy || '-' || mm || '-' || dd
  end

  return ymd

/* end ComposeDate */

/*=== Initialize() Initialize globals ===*/

Initialize: procedure expose (Globals)
  call SetCmdName
  call LoadRexxUtil
  gEnv = 'OS2ENVIRONMENT'
  return

/* end Initialize */

/*=== ScanArgsInit() ScanArgs initialization exit routine ===*/

ScanArgsInit: procedure expose (Globals) cmdTail swCtl keepQuoted
  /* Preset defaults */
  gDbgLvl = 0				/* Display debug messages */
  gVerbose = 0				/* Verbose messages */
  gArgList.0 = 0			/* Reset arg count */
  keepQuoted = 1			/* Set to 1 to keep arguments quoted */
  return

/* end ScanArgsInit */

/*=== ScanArgsSwitch() ScanArgs switch option exit routine ===*/

ScanArgsSwitch: procedure expose (Globals) curSw curSwArg
  select
  when curSw == 'd' then
    gDbgLvl = gDbgLvl + 1
  when curSw == 'h' | curSw == '?' then
    call ScanArgsHelp
  when curSw == 'v' then
    gVerbose = 1
  when curSw == 'V' then do
    say gCmdName gVersion
    exit
  end
  otherwise
    call ScanArgsUsage 'switch '''curSw''' unexpected'
  end /* select */

  return

/* end ScanArgsSwitch */

/*=== ScanArgsArg() ScanArgs argument option exit routine ===*/

ScanArgsArg: procedure expose (Globals) curArg
  i = gArgList.0 + 1
  gArgList.i = curArg
  gArgList.0 = i
  return

/* end ScanArgsArg */

/*=== ScanArgsTerm() ScanArgs scan end exit routine ===*/

ScanArgsTerm: procedure expose (Globals)
  if gArgList.0 = 0 then do
    gArgList.1 = '*.pdf'
    gArgList.0 = 1
  end
  return

/* end ScanArgsTerm */

/*=== ScanArgsHelp() Display ScanArgs usage help exit routine ===*/

ScanArgsHelp:
  say
  say 'Adjust file timestamp to match date embedded in file name.'
  say
  say 'Usage:' gCmdName '[-d] [-h] [-v] [-V] [-?] [filename]...'
  say
  say '  -d         Enable debug logic, repeat for more verbosity'
  say '  -h -?      Display this message'
  say '  -v         Enable verbose output'
  say '  -V         Display version number and quit'
  say
  say '  filename   Files to adjust, wildcards OK, defaults to *.pdf'
  exit 255

/* end ScanArgsHelp */

/*=== ScanArgsUsage(message) Report Scanargs usage error exit routine ===*/

ScanArgsUsage:
  parse arg msg
  say
  if msg \== '' then
    say msg
  say 'Usage:' gCmdName '[-d] [-h] [-v] [-V] [-?] [filename]...'
  exit 255

/* end ScanArgsUsage */

/*==============================================================================*/
/*=== SkelRexxFunc standards - Delete unused - Move modified above this mark ===*/
/*==============================================================================*/

/*=== AskYNQ([prompt][, noskip]) returns 0=Yes, 1=No, 2=Quit, skips line unless noskip ===*/

AskYNQ: procedure
  parse arg msg, noskip

  /* Take focus or fail if cannot match window title - assumes running 4OS2 */
  signal off Error
  /* Map 1st left bracket to wild card - [pid] seems to confuse activate */
  '@if defined _WINTITLE activate "%@replace[[,*,%_WINTITLE]"'
  signal on Error

  /* Skip line unless suppressed by noskip arg - any non-zero value requests noskip */
  if noskip = '' | noskip = 0 then
    call lineout 'STDERR', ''

  if msg == '' then
    msg = 'Continue'
  call charout 'STDERR', msg '(y/n/q) ? '
  do forever
    key = translate(SysGetKey('NOECHO'))
    if key == 'Y' | key == 'N' then do
      call lineout 'STDERR', key
      if key == 'Y' then
	ynq = 0
      else
	ynq = 1
      leave
    end
    if key == 'Q' | c2x(key) == '1B' then do
      call lineout 'STDERR', ''
      ynq = 2
      leave
    end
  end /* forever */
  return ynq

/* end AskYNQ */

/*=== DbgMsg([minLvl, ]message,...) Optionally write multi-line message to STDERR ===*/

/**
 * Write message if gDbgLvl >= minLvl
 * @param minLvl defaults to 1 if omitted
 * @returns true if message written
 */

DbgMsg: procedure expose (Globals)
  minLvl = arg(1)
  if datatype(minLvl, 'W') then
    start = 2
  else do
    minLvl = 1
    start = 1
  end
  if gDbgLvl >= minLvl then do
    do i = start to arg()
      msg = arg(i)
      if msg \== '' then
	msg = ' *' msg
      call lineout 'STDERR', msg
    end
  end
  return gDbgLvl >= minLvl

/* end DbgMsg */

/*==========================================================================*/
/*=== SkelRexx standards - Delete unused - Move modified above this mark ===*/
/*==========================================================================*/

/*=== Die([message,...]) Write multi-line message to STDERR and die ===*/

Die:
  call lineout 'STDERR', ''
  do i = 1 to arg()
    call lineout 'STDERR', arg(i)
  end
  call lineout 'STDERR', gCmdName 'aborting at line' SIGL || '.'
  call beep 200, 300
  call SysSleep 2
  exit 254

/* end Die */

/*=== Error() Set gErrCondition; report to STDOUT; trace and exit or return if called ===*/

Error:
  say
  parse source . . cmd
  gErrCondition = condition('C')
  say gErrCondition 'signaled at line' SIGL 'of' cmd || '.'
  if condition('D') \== '' then
    say 'REXX reason =' condition('D') || '.'
  if gErrCondition == 'SYNTAX' & symbol('RC') == 'VAR' then
    say 'REXX error =' RC '-' errortext(RC) || '.'
  else if symbol('RC') == 'VAR' then
    say 'RC =' RC || '.'
  say 'Source =' sourceline(SIGL)

  if condition('I') \== 'CALL' | gErrCondition == 'NOVALUE' | gErrCondition == 'SYNTAX' then do
    trace '?A'
    say 'Enter REXX commands to debug failure.  Press enter to exit script.'
    nop
    if symbol('RC') \== 'VAR' then
      RC = 255
    exit RC
  end

  return

/* end Error */

/*=== Halt() Report HALT condition to STDOUT and exit ===*/

Halt:
  say
  parse source . . cmd
  say condition('C') 'signaled at' cmd 'line' SIGL || '.'
  say 'Source =' sourceline(SIGL)
  say 'Sleeping for 2 seconds...'
  call SysSleep 2
  exit 253

/* end Halt */

/*=== LoadRexxUtil() Load RexxUtil functions ===*/

LoadRexxUtil:
  if RxFuncQuery('SysLoadFuncs') then do
    call RxFuncAdd 'SysLoadFuncs', 'REXXUTIL', 'SysLoadFuncs'
    if RESULT then
      call Die 'Cannot load SysLoadFuncs.'
    call SysLoadFuncs
  end
  return

/* end LoadRexxUtil */

/*=== ScanArgs(cmdLine) Scan command line ===*/

ScanArgs: procedure expose (Globals)

  /* Calls user exits to process arguments and switches */

  parse arg cmdTail
  cmdTail = strip(cmdTail)

  call ScanArgsInit

  /* Ensure optional settings initialized */
  if symbol('SWCTL') \== 'VAR' then
    swCtl = ''				/* Switches that take args, append ? if optional */
  if symbol('KEEPQUOTED') \== 'VAR' then
    keepQuoted = 0			/* Set to 1 to keep arguments quoted */

  /* Scan */
  curArg = ''				/* Current arg string */
  curSwList = ''			/* Current switch list */
  /* curSwArg = '' */			/* Current switch argument, if needed */
  noMoreSw = 0				/* End of switches */

  do while cmdTail \== '' | curArg \== '' | curSwList \== ''

    /* If arg buffer empty, refill */
    if curArg == '' then do
      qChar = left(cmdTail, 1)		/* Remember quote */
      if \ verify(qChar,'''"', 'M') then
	parse var cmdTail curArg cmdTail	/* Not quoted */
      else do
	/* Arg is quoted */
	curArg = ''
	do forever
	  /* Parse dropping quotes */
	  parse var cmdTail (qChar)quotedPart(qChar) cmdTail
	  curArg = curArg || quotedPart
	  /* Check for escaped quote within quoted string (i.e. "" or '') */
	  if left(cmdTail, 1) \== qChar then do
	    cmdTail = strip(cmdTail)	/* Strip leading whitespace */
	    leave			/* Done with this quoted arg */
	  end
	  curArg = curArg || qChar	/* Append quote */
	  if keepQuoted then
	    curArg = curArg || qChar	/* Append escaped quote */
	  parse var cmdTail (qChar) cmdTail	/* Strip quote */
	end /* do forever */
	if keepQuoted then
	  curArg = qChar || curArg || qChar	/* requote */
      end /* if quoted */
    end /* if curArg empty */

    /* If switch buffer empty, refill */
    if curSwList == '' & \ noMoreSw then do
      if left(curArg, 1) == '-' & curArg \== '-' then do
	if curArg == '--' then
	  noMoreSw = 1
	else
	  curSwList = substr(curArg, 2)	/* Remember switch string */
	curArg = ''			/* Mark empty */
	iterate				/* Refill arg buffer */
      end /* if switch */
    end /* if curSwList empty */

    /* If switch in progress */
    if curSwList \== '' then do
      curSw = left(curSwList, 1)	/* Next switch */
      curSwList = substr(curSwList, 2)	/* Drop from pending */
      /* Check switch allows argument, avoid matching ? */
      if pos(curSw, translate(swCtl,,'?')) \= 0 then do
	if curSwList \== '' then do
	  curSwArg = curSwList		/* Use rest of switch string for switch argument */
	  curSwList = ''
	end
	else if curArg \== '' & left(curArg, 1) \== '-' then do
	  curSwArg = curArg		/* Arg string is switch argument */
	  curArg = ''			/* Mark arg string empty */
	end
	else if pos(curSw'?', swCtl) = 0 then
	  call ScanArgsUsage 'Switch "-' || curSw || '" requires an argument'
	else
	  curSwArg = ''			/* Optional arg omitted */
      end

      call ScanArgsSwitch		/* Passing curSw and curSwArg */
      drop curSwArg			/* Must be used by now */
    end /* if switch */

    /* If arg */
    else if curArg \== '' then do
      noMoreSw = 1
      call ScanArgsArg			/* Passing curArg */
      curArg = ''
    end

  end /* while not done */

  call ScanArgsTerm

  return

/* end ScanArgs */

/*=== SetCmdName() Set gCmdName to short script name ===*/

SetCmdName: procedure expose (Globals)
  parse source . . cmd
  cmd = filespec('N', cmd)		/* Chop path */
  c = lastpos('.', cmd)
  if c > 1 then
    cmd = left(cmd, c - 1)		/* Chop extension */
  gCmdName = translate(cmd, xrange('a', 'z'), xrange('A', 'Z'))	/* Lowercase */
  return

/* end SetCmdName */

/* The end */
